﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Configuration;
using System.Linq;
using System.Web;
using System.Web.Hosting;
using System.Xml.Linq;

public class UrlRewriter : IHttpHandler
{
	private SayuminDataContext db;
	private List<string> virtualDirs;

	public bool IsReusable
	{
		get
		{
			return false;
		}
	}

	public UrlRewriter()
	{
		this.db = new SayuminDataContext();
		this.virtualDirs = new List<string>()
		{
			"~/radioshow/",
			"~/episode/",
			"~/news/"
		};
	}

	public void ProcessRequest(System.Web.HttpContext context)
	{
		var path = context.Request.Url.AbsolutePath;
		var isVirtual = this.isPathVirtual(path);

		if (!isVirtual)
		{
			return;
		}

		var realPath = this.getRealPath(path);

        string realPage;
        if (realPath == null)
        {
            realPath = "~/error/404.aspx";
            realPage = realPath;
        }
        else
        {
            realPage = realPath.Substring(0, realPath.IndexOf('?'));
        }

        string queryString = context.Request.Url.Query;
        if (false == string.IsNullOrEmpty(queryString))
        {
            queryString = queryString.Replace("?", "");
            realPath += "&" + queryString;
        }

		context.RewritePath(realPath, true);
		context.Server.Transfer(realPage);
	}

	private bool isPathVirtual(string basePath)
	{
		string appPath = VirtualPathUtility.ToAppRelative(basePath);

		foreach (var dir in this.virtualDirs)
		{
			if (appPath.StartsWith(dir))
			{
				return true;
			}
		}

		return false;
	}

	private string getRealPath(string basePath)
	{
		string searchPath = basePath.Replace(".aspx", "");
		string[] searchArray = searchPath.Split('/');
		int depth = searchArray.Length;

		string realPath = null;

		switch (searchArray[1])
		{
			case "radioshow":
				// /radioshows/<show>.aspx
				realPath = this.getRadioShowPath(searchArray[2]);
				break; ;

			case "episode":
				realPath = this.getEpisodePath(searchArray);
				// /episodes/<show>/<date>.aspx
				break; ;

			case "news":
				// /news/date/title.aspx
				realPath = this.getNewsPath(searchArray);
				break; ;

		}

		return realPath;
	}

	private RadioShow getRadioShow(string pathComponent)
	{
		var radioshow = (from rs in db.RadioShows
						 where rs.Folder == pathComponent
						 select rs).SingleOrDefault();
		return radioshow;
	}

	private DateTime getDate(string dateComponent)
	{
		string[] dateComponents = dateComponent.Split('-');
		int year = int.Parse(dateComponents[0]);
		int month = int.Parse(dateComponents[1]);
		int date = int.Parse(dateComponents[2]);

		var episodeDate = new DateTime(year, month, date);
		return episodeDate;
	}

	private string getRadioShowPath(string pathComponent)
	{
		var radioshow = this.getRadioShow(pathComponent);
		if (radioshow == null)
		{
			return null;
		}

		return string.Format("/radioshow.aspx?ID={0}", radioshow.ID);
	}

	private string getEpisodePath(string[] pathComponents)
	{
		var radioshow = this.getRadioShow(pathComponents[2]);
		if (radioshow == null)
		{
			return null;
		}

		var episodeDate = this.getDate(pathComponents[3]);

		var episode = (from e in db.Episodes
					   where e.RadioShow == radioshow
					   && e.Date == episodeDate
					   select e).FirstOrDefault(p => p.ID > 0);

		if (episode == null)
		{
			return null;
		}

		return string.Format("/episode.aspx?ID={0}", episode.ID);
	}

	private string getNewsPath(string[] pathComponents)
	{
		var newsDate = this.getDate(pathComponents[2]);

		var newsItems = from n in db.News
					    where n.Date.Date == newsDate
					    select n;

		var newsItem = this.findItem(newsItems);

		if (newsItem == null)
		{
			return null;
		}

		return string.Format("/news.aspx?ID={0}", newsItem.ID);
	}

	private News findItem(IQueryable<News> newsItems)
	{
		foreach (var newsItem in newsItems)
		{
			string link = Globals.GetUrl(newsItem);

			if (HttpContext.Current.Request.Url.AbsolutePath == link)
			{
				return newsItem;
			}
		}

		return null;
	}
}
