using System;
using System.Text.RegularExpressions;

namespace WebToolkit
{
    /// <summary>
    /// Class containing common useful Url-related functions
    /// </summary>
    public static class UrlTools
    {
        /// <summary>
        /// Creates a valid HTML-link based on a URL and fills in
        /// the blanks
        /// </summary>
        /// <param name="link">String containing URL</param>
        /// <returns>String containing HTML-formatted link</returns>
        public static string FormatLink(object link)
        {
            string formated;
            if (link is DBNull)
            {
                formated = "";
            }
            else
            {
                string sLink = (string)link;
                string http = FormatHttpLink(sLink);
                string plain = FormatHttpLessLink(sLink);

                formated = "<a href=\"" + http + "\">" + plain + "</a>";
            }
            return formated;
        }

        private static string FormatHttpLink(string link)
        {
            if (!link.StartsWith("http://"))
            {
                link = "http://" + link;
            }
            return link;
        }

        private static string FormatHttpLessLink(string link)
        {
            if (link.StartsWith("http://"))
            {
                link = link.Substring(7);
            }
            return link;
        }

        /// <summary>
        /// Filters a text so the remaining characters are URL-safe
        /// </summary>
        /// <param name="HeaderText">text to cleaned</param>
        /// <returns>reformatted and cleaned text suitable for a URL</returns>
        public static string FilterAndClean(string HeaderText)
        {
            string text = HeaderText;
            string clean = "?\"():;";

            foreach (char c in clean)
            {
                text = text.Replace(c, '!');
            }
            text = text.Replace("!", "");
			text = text.Replace("'", "");

            Regex rxClean = new Regex("([^A-Za-z0-9_!])");
            text = rxClean.Replace(text, "_");

            return text;
        }
    }
}
