﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;

public partial class CommentViewer : System.Web.UI.UserControl
{
    private IEnumerable<Comment> comments;

    public object CommentObject { get; set; }

    protected void Page_Load(object sender, EventArgs e)
    {
        this.getComments();

        bool hasComments = (this.comments != null) && this.comments.Count() != 0;

        this.pCommentList.Visible = hasComments;
        this.pNoComments.Visible = !hasComments;

        if (hasComments)
        {
            this.addComments();
        }

        // handle commenting
        string cssElement = Page.IsPostBack ? "addComment" : "composeComment";
        string styleText = string.Format("<style type=\"text/css\">#{0}", cssElement);
        styleText += " { display: none; overflow-y: hidden; }</style>";

        var masterPage = (SayuminMaster)Page.Master;
        var cph = (ContentPlaceHolder)masterPage.FindControl("head");
        var litStyle = new Literal();
        litStyle.Text = styleText;
        cph.Controls.Add(litStyle);

        // fill inn cookie-info
        if (!Page.IsPostBack)
        {
            try
            {
                HttpCookie cookie = Request.Cookies["www.sayumin.org"];
                if (cookie != null)
                {
                    this.txtName.Text = cookie.Values["PostName"];
                    this.txtWebsite.Text = cookie.Values["HomePage"];
                }
            }
            catch (Exception) { }
        }
    }

    private void getComments()
    {
        if (this.CommentObject == null)
        {
            return;
        }

        var db = new SayuminDataContext();
		string linkRSS = "/rss/Comments.ashx?{0}ID={1}";

        var episodeComment = this.CommentObject as Episode;
        if (episodeComment != null)
        {
            this.comments = from c in db.Comments
                            where c.EpisodeID == (int?)episodeComment.ID
                            orderby c.PostDate descending
                            select c;
			this.lnkCommentRSS.HRef = string.Format(linkRSS, "Episode", episodeComment.ID);
			return;
        }

        var radioshowComment = this.CommentObject as RadioShow;
        if (radioshowComment != null)
        {
            this.comments = from c in db.Comments
                            where c.RadioshowID == (int?)Convert.ToInt32(radioshowComment.ID)
                            orderby c.PostDate descending
                            select c;
			this.lnkCommentRSS.HRef = string.Format(linkRSS, "Radioshow", radioshowComment.ID);
			return;
        }

        var newsComment = this.CommentObject as News;
        if (newsComment != null)
        {
            this.comments = from c in db.Comments
                            where c.NewsID == (int?)newsComment.ID
                            orderby c.PostDate descending
                            select c;
			this.lnkCommentRSS.HRef = string.Format(linkRSS, "News", newsComment.ID);
			return;
        }
    }

    public void addComments()
    {
        foreach (var comment in this.comments)
        {
            var commentForm = (CommentForm)Page.LoadControl("~/CommentForm.ascx");
            commentForm.Comment = comment;
            commentForm.QuoteHandler += new CommandEventHandler(this.quote);

            this.pCommentList.Controls.Add(commentForm);
        }
    }

    private void cleanForm()
    {
        this.txtName.Text = this.txtName.Text.Trim();
        this.txtWebsite.Text = this.txtWebsite.Text.Trim();
        this.txtComment.Text = this.txtComment.Text.Trim();
    }

    private Comment getCommentFromForm()
    {
        var comment = new Comment()
        {
            Nick = this.txtName.Text,
            PostDate = DateTime.Now,
            Website = this.txtWebsite.Text,
            Text = this.txtComment.Text
        };

        var episodeComment = this.CommentObject as Episode;
        var radioshowComment = this.CommentObject as RadioShow;
        var newsComment = this.CommentObject as News;

        if (episodeComment != null)
        {
            comment.EpisodeID = episodeComment.ID;
        }
        else if (radioshowComment != null)
        {
            comment.RadioshowID = (int?) Convert.ToInt32(radioshowComment.ID);
        }
        else if (newsComment != null)
        {
            comment.NewsID = newsComment.ID;
        }

        return comment;
    }

    protected void lnkAdd_Click(object sender, EventArgs e)
    {
        this.cleanForm();

		if (!Page.IsValid)
		{
			return;
		}

        var c = this.getCommentFromForm();

        var db = new SayuminDataContext();
        db.Comments.InsertOnSubmit(c);
        db.SubmitChanges();

        try
        {
            HttpCookie cookie = new HttpCookie("www.sayumin.org");
            cookie.Values.Add("PostName", c.Nick);
            cookie.Values.Add("HomePage", c.Website);
            cookie.Expires = DateTime.Now.AddDays(31);
            Response.Cookies.Add(cookie);
        }
        catch (Exception) { }

        this.Page.Response.Redirect(this.Page.Request.Url.AbsoluteUri + "#comments");
    }

    protected void lnkPreview_Click(object sender, EventArgs e)
    {
        this.cleanForm();
        var c = this.getCommentFromForm();

        var commentForm = (CommentForm)Page.LoadControl("~/CommentForm.ascx");
        commentForm.Comment = c;
        commentForm.QuoteHandler += new CommandEventHandler(this.quote);

        this.pCommentPreview.Controls.Add(commentForm);
    }

    protected void quote(object sender, CommandEventArgs e)
    {
        int commentID = int.Parse((string) e.CommandArgument);

        // do not quote preview comments
        if (commentID == 0)
        {
            return;
        }

        var comment = (from c in this.comments
                       where c.ID == commentID
                       select c).Single();

        string quoteText = string.Format("[quote][b]{0} said:[/b]\n{1}\n[/quote]\n\n", comment.Nick, comment.Text);
        this.txtComment.Text += quoteText;

        this.txtComment.Focus();
    }
}
