﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

namespace Sayumin.Common
{
    public class InferringScanner : BaseScanner
    {
        protected bool inferYear;
        protected bool inferMonth;
        protected bool inferDate;

        /// <summary>
        /// Scanner class which infers date-values based on reported uploaded date
        /// Behvaiour is configurable trough the infer=year,month,date config setting.
        /// </summary>
        public InferringScanner() : base()
        {
        }

        public override void  InitSettings(string Settings)
        {
 	        base.InitSettings(Settings);

            if (_settings.Keys.Contains("infer"))
            {
                string infer = _settings["infer"].ToLower();
                string[] values = infer.Split(new char[] { ',' });

                inferYear = values.Contains("year");
                inferMonth = values.Contains("month");
                inferDate = values.Contains("date");
            }
        }

        protected override bool MatchDate(FileEntry Entry, DateTime EpisodeDate)
        {
            DateTime detectedDate = DateTime.MinValue;
            Match m = Regex.Match(Entry.Description, _settings["dateformat"]);

            if (m.Success)
            {
                string yy = inferYear ? EpisodeDate.Year.ToString() : string.Empty;
                string mm = inferMonth ? EpisodeDate.Month.ToString() : string.Empty;
                string dd = inferDate ? EpisodeDate.Day.ToString() : string.Empty;

                if (false == inferYear)
                {
                    yy = m.Groups["Y"].Value;
                }
                if (false == inferMonth)
                {
                    mm = m.Groups["M"].Value;
                }
                if (false == inferDate)
                {
                    dd = m.Groups["D"].Value;
                }

                if (yy.Length == 2)
                {
                    yy = "20" + yy;
                }


                string dateString = string.Format("{0}-{1}-{2}", yy, mm, dd);
                detectedDate = DateTime.Parse(dateString);
            }

            return detectedDate == EpisodeDate;
        }
    }
}
