﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Linq;
using System.ServiceProcess;
using System.Text;
using Sayumin.Common;
using System.Threading;

namespace Sayumin.Common
{
    public partial class SayuminBaseService : ServiceBase
    {
        private Thread _mainThread;

        public SayuminBaseService()
        {
            InitializeComponent();
            AutoLog = false;
        }

        protected bool _exitSignaled;
        protected ILogModule _logModule;

        public void Start()
        {
            OnStart(null);
        }

        protected override void OnStart(string[] args)
        {
            MultiLogger mlogger = new MultiLogger();
            mlogger.Modules.Add(new ConsoleLogger());
            mlogger.Modules.Add(new EventLogLogger(ServiceName));
            mlogger.Modules.Add(new DbLogger(ServiceName));

            _logModule = mlogger;

            _logModule.Write("Service starting.");

            _mainThread = new Thread(delegate()
            {
                while (false == _exitSignaled)
                {
                    try
                    {
                        DoWork();
                    }
                    catch (Exception ex)
                    {
                        // in case any logging module is a db-logger and db-failure
                        try
                        {
                            if (ex is ThreadAbortException)
                            {
                                _logModule.Write("Main thread did not shutdown cleanly.");
                                break;
                            }

                            _logModule.WriteError("Error in main loop", ex);
                        }
                        catch (Exception ex2)
                        {
                            ILogModule logger = new EventLogLogger(ServiceName);
                            logger.WriteError("Errors was encountered when logging events", ex2);
                            logger.WriteError("Original error", ex);
                        }
                    }
                }
            });
            _mainThread.Start();

            _logModule.Write("Service started.");
        }

        protected override void OnStop()
        {
            _logModule.Write("Service stopping.");
            StopThread();
        }

        protected override void OnShutdown()
        {
            base.OnShutdown();

            _logModule.Write("Shutdown signalled. Service stopping.");
            StopThread();
        }

        protected virtual void StopThread()
        {
            _exitSignaled = true;

            // give the thread time to die off on its own.
            Thread.Sleep(2000);

            if (_mainThread.ThreadState != System.Threading.ThreadState.Stopped)
            {
                _mainThread.Abort();
            }
        }

        protected virtual void DoWork()
        {
            // do nothing.
        }

        /// <summary>
        /// Starts a service in correct mode depending on if it is 
        /// </summary>
        /// <param name="Service"></param>
        public static void StartService(SayuminBaseService Service)
        {
            if (Environment.UserInteractive)
            {
                Console.WriteLine("Starting in Interactive-mode. Press ctrl-c to force exit.");

                Service.Start();

                while (true)
                {
                    Thread.Sleep(1000);
                }

            }
            else
            {
                ServiceBase[] ServicesToRun;
                ServicesToRun = new ServiceBase[] 
			    { 
				    Service 
			    };
                ServiceBase.Run(ServicesToRun);
            }
        }
    }
}
