﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceProcess;
using System.Text;
using Sayumin.Common;

namespace Sayumin.QueueGenerator
{
    static class Program
    {
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        static void Main(string[] args)
        {
            if (args.Length != 0)
            {
                AddForSpecificDates(args);
                return;
            }

            SayuminBaseService service = new QueueGeneratorService();
            SayuminBaseService.StartService(service);
        }

        static void AddForSpecificDates(string[] dateStrings)
        {
            Console.WriteLine("Sayumin QueueGenerator starting in manual mode.");
            Console.WriteLine("");

            QueueItemHandler qih = new QueueItemHandler();

            foreach (string dateString in dateStrings)
            {
                DateTime scheduleDate = default(DateTime);

                if (DateTime.TryParse(dateString, out scheduleDate))
                {
                    Console.Write("Adding for date {0:yyyy-MM-dd}: ", scheduleDate);
                    string[] shows = qih.GenerateFor(scheduleDate);

                    if (shows.Length == 0)
                    {
                        Console.WriteLine("None.");
                    }
                    else
                    {
                        Console.WriteLine(Concat(shows, ", ") + ".");
                    }
                }
                else
                {
                    Console.WriteLine("Error parsing parameter '{0}' as a valid date.", dateString);
                }
            }

            Console.WriteLine("");
            Console.WriteLine("Done.");
        }

        static string Concat(string[] args, string seperator)
        {
            StringBuilder sb = new StringBuilder();
            bool first = true;

            foreach (string text in args)
            {
                if (false == first)
                {
                    sb.Append(seperator);
                }
                sb.Append(text);
                first = false;
            }

            return sb.ToString();
        }
    }
}
