﻿using System;
using System.Text;
using System.Threading;
using Sayumin.Common;

namespace Sayumin.QueueGenerator
{
    public partial class QueueGeneratorService : SayuminBaseService
    {
        public QueueGeneratorService() : base()
        {
            ServiceName = "Sayumin.QueueGenerator";
        }

        protected override void DoWork()
        {
            QueueItemHandler qih = new QueueItemHandler();
            DateTime queueUpdated = DateTime.MinValue;

            int scansPerMinute = 1;
            int totalSleepLength = (int)(60000 / scansPerMinute); // milliseconds

            int sleepStateLength = 1000; // milliseconds
            int sleepsPerScan = totalSleepLength / sleepStateLength;

            while (false == _exitSignaled)
            {
                DateTime now = DateTime.Now;

                // repopulate queue with new items after midnight
                if (now.Date > queueUpdated.Date)
                {
                    _logModule.Write("Populating queue.");
                    string[] showsAdded = qih.GenerateFor(now.Date);

                    _logModule.Write("Show(s) added: " + Concat(showsAdded, ", "));

                    queueUpdated = now.Date;
                }

                // check if items should be activated
                string[] showsActivated = qih.CheckState();

                if (showsActivated.Length != 0)
                {
                    _logModule.Write("Show(s) activated: " + Concat(showsActivated, ", "));
                }

                for (int i = 0; i < sleepsPerScan; i++)
                {
                    Thread.Sleep(sleepStateLength);

                    if (true == _exitSignaled)
                    {
                        break;
                    }
                }
            }

            // exit;
        }

        public string Concat(string[] items, string seperator)
        {
            bool first = true;

            StringBuilder sb = new StringBuilder();

            foreach (string item in items)
            {
                if (false == first)
                {
                    sb.Append(seperator);
                }
                sb.Append(item);
                first = false;
            }

            return sb.ToString();
        }
    }
}
