﻿using System;
using System.Collections.Generic;
using System.Data.SqlClient;
using System.Linq;
using Sayumin.Common;
using Sayumin.Common.DAC;

namespace Sayumin.QueueGenerator
{
    public class QueueItemHandler
    {
        public QueueItemHandler()
        {
        }

        /// <summary>
        /// Adds new items to the FileQueue-table with a status of "waiting" for the specified date.
        /// The schedule generated for date X may include episodes from prior dates based on schedule configuration.
        /// </summary>
        /// <param name="ScheduleDate">The date for which a download-schedule should be generated</param>
        public string[] GenerateFor(DateTime ScheduleDate)
        {
            string[] showsAddedToQueue = new string[0];

            using (SayuminDataContext db = new SayuminDataContext())
            {
                List<string> showsAdded = new List<string>();

                var query = from s in db.Schedules
                            where s.StartDay == (int)ScheduleDate.DayOfWeek
                            && s.IsActive == true
                            select s;

                foreach (Schedule schedule in query)
                {
                    SayuminDataContext db2 = new SayuminDataContext();

                    // find airdate for schedule
                    DateTime episodeDate = ScheduleDate;
                    while (episodeDate.DayOfWeek != schedule.DayOfWeekAired)
                    {
                        episodeDate = episodeDate.AddDays(-1);
                    }

                    FileQueue item = new FileQueue();
                    item.Attempts = 0;
                    item.EpisodeDate = episodeDate.Date;
                    item.RadioShowID = schedule.RadioShowID;
                    item.ScheduleID = schedule.ID;
                    item.StartTime = ScheduleDate.Date.Add(schedule.StartTime.TimeOfDay);
                    item.Status = QueueStatus.Waiting;

                    db2.FileQueues.InsertOnSubmit(item);

                    // duplicates is known to happen when run multiple times in one day, and on boot.
                    bool isDuplicate = db2.SubmitChangesAndIgnoreDuplicates();
                    if (false == isDuplicate)
                    {
                        showsAdded.Add(schedule.RadioShow.DisplayName);
                    }
                }


                showsAddedToQueue = showsAdded.ToArray();
            }

            return showsAddedToQueue;
        }

        /// <summary>
        /// Checks the state of existing, waiting items and activates them as needed.
        /// </summary>
        public string[] CheckState()
        {
            string[] showsActivated = new string[0];

            using (SayuminDataContext db = new SayuminDataContext())
            {
                var query = from fq in db.FileQueues
                            where fq.StartTime <= DateTime.Now
                            && fq.StatusNumber == (int)QueueStatus.Waiting
                            select fq;

                List<string> shows = new List<string>();
                foreach (FileQueue fq in query)
                {
                    fq.Status = QueueStatus.Active;
                    shows.Add(fq.RadioShow.DisplayName);
                }

                db.SubmitChanges();
                showsActivated = shows.ToArray();
            }

            return showsActivated;
        }
    }
}
