﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Sayumin.Common;

namespace Sayumin.Tests.Common
{
    /// <summary>
    /// Summary description for UnitTest1
    /// </summary>
    [TestClass]
    public class BaseSourceTest
    {
        IFileSource _source;
        public BaseSourceTest()
        {
            _source = new BaseSource();
        }

        #region vsTestCode

        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #endregion vsTestCode

        private void GetSiteData(string configString, int numEntries, int numColumns)
        {
            BaseSourceTest.GetSiteData(_source, configString, numEntries, numColumns);
        }

        public static void GetSiteData(IFileSource source, string configString, int numEntries, int numColumns)
        {
            IConfigurable configurableSource = source as IConfigurable;
            if (configurableSource != null)
            {
                configurableSource.InitSettings(configString);
            }

            DateTime check = DateTime.Now;
            FileEntry[] entries = source.GetEntries();

            bool updatedDate = check <= source.LastUpdated;
            Assert.IsTrue(updatedDate, "Date should be updated to be have a later value than before it was called.");

            Assert.AreEqual<int>(numEntries, entries.Length, "There should be " + numEntries.ToString() + " entries ready for analysis.");

            foreach (FileEntry entry in entries)
            {
                Assert.AreEqual<int>(numColumns, entry.Columns.Keys.Count, "Every entry should have " + numColumns + " additional columns.");

                bool containsRelativeComponents = entry.Url.Contains("./");
                Assert.IsFalse(containsRelativeComponents, "Every entry should contain absolute URLs with no relative components.");

                // substring used to avoid the http:// bit
                bool containsDoubleSlashes = entry.Url.Substring(7).Contains("//");
                Assert.IsFalse(containsDoubleSlashes, "Every entry should contain absolute URLs and no path resetting double-slashes.");
            }
        }

        [TestMethod]
        public void HelloTC() // just for general testing. discontinued
        {
            string configString = "url=http://test.hello.tc/uploader/exbfagir.php;encoding=UTF-8";
            int numEntries = 20;
            int numColumns = 2;

            GetSiteData(configString, numEntries, numColumns);
        }

        [TestMethod]
        public void BerryzKobou() // Berryz Koubou - Kiritsu! Rei! Chakuseki!
        {
            string configString = "url=http://berryz-kobo.jp/loda/upload.php;encoding=SJIS;tagdelimiter='";
            int numEntries = 50;
            int numColumns = 1;

            GetSiteData(configString, numEntries, numColumns);
        }

        [TestMethod]
        public void Konchan() // General purpose
        {
            string configString = "url=http://konchan.fam.cx/0412/hello/upload.php";
            int numEntries = 20;
            int numColumns = 4;

            GetSiteData(configString, numEntries, numColumns);
        }

        [TestMethod]
        public void Kumaroda() // Berryz Koubou - Kiritsu! Rei! Chakuseki!
        {
            string configString = "url=http://kumaiyurina.dyndns.org/YRNL/upload.php?gal=0";
            int numEntries = 20;
            int numColumns = 3;

            GetSiteData(configString, numEntries, numColumns);
        }

        [TestMethod]
        public void Momoroda() // Berryz Koubou - Kiritsu! Rei! Chakuseki!
        {
            string configString = "url=http://tugunaga.naruserv.net/momoti.php";
            int numEntries = 20;
            int numColumns = 4;

            GetSiteData(configString, numEntries, numColumns);
        }

        [TestMethod]
        public void Himeji() // Five Stars - Matsuura
        {
            string configString = "url=http://rainbow.blogsite.org/cgi-bin/upload.php";
            int numEntries = 20;
            int numColumns = 2;

            GetSiteData(configString, numEntries, numColumns);
        }

        [TestMethod]
        public void google723roda() // Five Stars - Abe
        {
            string configString = "url=http://nacci723.gotdns.com:723/natsumi/upload.php;referrer=http://zzpyaym.hp.infoseek.co.jp/natsumi/";
            int numEntries = 20;
            int numColumns = 4;

            GetSiteData(configString, numEntries, numColumns);
        }

        // non-operational
        [TestMethod]
        public void PappyReina() // Five Stars - Tanaka :  
        {
            // TODO: ERROR : times out. Test later.
            string configString = "url=http://ng2.orz.hm/ReinaT/pappy_reina.php";
            int numEntries = 20;
            int numColumns = 2;

            GetSiteData(configString, numEntries, numColumns);
        }

        [TestMethod]
        public void Cuteroda() // Cutie paradise, I Me Maimi
        {
            string configString = "url=http://hekocute.dip.jp/large/upload.php";
            int numEntries = 20;
            int numColumns = 2;

            GetSiteData(configString, numEntries, numColumns);
        }

        [TestMethod]
        public void Maimiroda() // I me Maimi
        {
            string configString = "url=http://maimi.hello.tc/up/upload55.php;encoding=UTF-8";
            int numEntries = 20;
            int numColumns = 2;

            GetSiteData(configString, numEntries, numColumns);
        }

        [TestMethod]
        public void Chiisaaroda() // I me Maimi
        {
            string configString = "url=http://cocoro.blogsite.org/cgi-bin/upload.php";
            int numEntries = 20;
            int numColumns = 2;

            GetSiteData(configString, numEntries, numColumns);
        }

        [TestMethod]
        public void RainbowRoda() // Gyuu to naito, Space Diva - Song 4 Anniversary
        {
            string configString = "url=http://hm723.sakura.ne.jp/melon/index.php";
            int numEntries = 30; // might not be permanent
            int numColumns = 5;

            GetSiteData(configString, numEntries, numColumns);
        }

        [TestMethod]
        public void Miup() // Maititaimu, Sunday otana lab
        {
            string configString = "url=http://miuna.no-ip.org/up/miup.php";
            int numEntries = 20; // might not be permanent
            int numColumns = 3;

            GetSiteData(configString, numEntries, numColumns);
        }

        [TestMethod]
        public void DoctorLoader() // Gatan Guts
        {
            string configString = "url=http://viyuden.hello.tc/doctor/dddddddd.php;encoding=UTF-8";
            int numEntries = 52;
            int numColumns = 2;

            GetSiteData(configString, numEntries, numColumns);
        }

        [TestMethod]
        public void TaiwanRoda() // Goto Maki J-Wave Sweet Black Girls
        {
            string configString = "url=http://momusume.no-ip.org/mmintw.php";
            int numEntries = 20;
            int numColumns = 3;

            GetSiteData(configString, numEntries, numColumns);
        }
    }
}
