﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Sayumin.Common.DAC;
using Sayumin.Common;

namespace Sayumin.Tests.Common
{
    /// <summary>
    /// Summary description for ModuleLoadingTests
    /// </summary>
    [TestClass]
    public class ModuleLoadingTests
    {
        SayuminDataContext db;

        public ModuleLoadingTests()
        {
            db = new SayuminDataContext();
        }

        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        [TestMethod]
        public void LoadSources()
        {
            var query = from s in db.Sources
                        select s;

            foreach (Source source in query)
            {
                IFileSource module = source.GetInstance();
                Assert.IsNotNull(module, "All modules should be configured to load properly.");

                IConfigurable configurableModule = module as IConfigurable;

                if (null != configurableModule)
                {
                    Exception caught = null;
                    try
                    {
                        configurableModule.InitSettings(source.Configuration);
                    }
                    catch (Exception ex)
                    {
                        caught = ex;
                    }

                    Assert.IsNull(caught, "No modules should crash by loading configuration");
                }
            }
        }

        [TestMethod]
        public void LoadScanners()
        {
            var query = from s in db.Scanners
                        select s;

            foreach (Scanner scanner in query)
            {
                IRadioScanner module = scanner.GetInstance();
                Assert.IsNotNull(module, "All modules should be configured to load properly.");
            }
        }

        [TestMethod]
        public void ScannerAndSourceConfiguration()
        {
            var query = from rss in db.RadioshowSources
                        select rss;

            foreach (RadioshowSource rss in query)
            {
                Source source = rss.Source;

                IFileSource sourceModule = source.GetInstance();
                Assert.IsNotNull(sourceModule, "All modules should be configured to load properly.");

                IConfigurable configurableModule = sourceModule as IConfigurable;

                if (null != configurableModule)
                {
                    Exception caught = null;
                    try
                    {
                        configurableModule.InitSettings(source.Configuration);
                    }
                    catch (Exception ex)
                    {
                        caught = ex;
                    }

                    Assert.IsNull(caught, "No modules should crash by loading configuration");
                }

                Scanner scanner = rss.Scanner;
                IRadioScanner radioscannerModule = scanner.GetInstance();

                configurableModule = radioscannerModule as IConfigurable;
                if (null != configurableModule)
                {
                    Exception caught = null;
                    try
                    {
                        configurableModule.InitSettings(rss.Configuration);
                    }
                    catch (Exception ex)
                    {
                        caught = ex;
                    }

                    Assert.IsNull(caught, "No modules should crash by loading configuration");
                }

            }
        }
    }
}
