﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Collections;

namespace Sayumin.Tests.SourceConfig
{
    public partial class MainForm : Form, IScheduleView
    {
        public IScheduleController Controller;

        public MainForm()
        {
            InitializeComponent();
        }

        /*
         * public properties
         */

        public string[] RecentDates
        {
            set
            {
                updateRecentDates(value);
            }
        }

        public DayOfWeek[] ActiveDays
        {
            get
            {
                return getActiveDays();
            }
            set
            {
                setActiveDays(value);
            }
        }

        public bool IsActive
        {
            get
            {
                return checkBoxActive.Checked;
            }
            set
            {
                checkBoxActive.Checked = value;
            }
        }

        public DateTime DatePicked
        {
            get
            {
                return dateTimePickerSchedule.Value;
            }
            set
            {
                dateTimePickerSchedule.Value = value;
            }
        }

        public DateTime TimePicked
        {
            get
            {
                return DateTime.Parse(textBoxStartTime.Text);
            }
            set
            {
                textBoxStartTime.Text = value.ToString("hh:mm");
            }
        }

        public string Description
        {
            get { return textBoxDescription.Text; }
        }

        public string Extension
        {
            get { return textBoxExtension.Text; }
        }

        public string Orig
        {
            get { return textBoxOrig.Text; }
        }

        public IEnumerable TestResults
        {
            set { setTestResults(value); }
        }

        public IEnumerable Filters
        {
            set { setFilters(value); }
        }

        /*
         * protected methods
         */

        protected void updateRecentDates(string[] RecentDates)
        {
            listBoxRecentDates.Items.Clear();
            listBoxRecentDates.Items.AddRange(RecentDates);
        }

        protected DayOfWeek[] getActiveDays()
        {
            List<DayOfWeek> days = new List<DayOfWeek>();
            for (int i = 0; i < checkedListBoxDays.Items.Count; i++)
            {
                bool isChecked = checkedListBoxDays.GetItemChecked(i);

                if (isChecked)
                {
                    DayOfWeek dayOfWeek = (DayOfWeek)((i + 1) % 7);
                    days.Add(dayOfWeek);
                }
            }
            return days.ToArray();
        }

        protected void setActiveDays(DayOfWeek[] Days)
        {
            for (int i = 0; i < checkedListBoxDays.Items.Count; i++)
            {
                DayOfWeek dayOfWeek = (DayOfWeek)((i + 1) % 7);

                bool isChecked = Days.Contains(dayOfWeek);

                checkedListBoxDays.SetItemChecked(i, isChecked);
            }
        }

        protected void setTestResults(IEnumerable data)
        {
            dataGridViewTestResults.DataSource = data;
        }

        protected void setFilters(IEnumerable filters)
        {
            dataGridViewFilters.DataSource = filters;
        }

        /*
         * event handlers
         */

        private void dateTimePickerSchedule_ValueChanged(object sender, EventArgs e)
        {
            Controller.DateChosen();
        }

        private void checkedListBoxDays_SelectedValueChanged(object sender, EventArgs e)
        {
            Controller.DayChosen();
        }

        private void textBoxStartTime_Leave(object sender, EventArgs e)
        {
            Controller.StartTimeChanged();
        }

        private void checkBoxActive_CheckedChanged(object sender, EventArgs e)
        {
            Controller.EnabledChosen();
        }

        private void buttonTest_Click(object sender, EventArgs e)
        {
            Controller.TestFilters();
        }

        private void buttonAdd_Click(object sender, EventArgs e)
        {
            Controller.AddFilters();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void newToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Controller.New();
        }

        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Controller.Open();
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Controller.Save();
        }
    }
}
