﻿using System;
using System.Collections;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Xml.Linq;

public partial class SayuminMaster : System.Web.UI.MasterPage
{
    protected string[] themes = new string[]
    {
        "/style/theme.css",  // classic
        "/style/theme2.css", // sleepy white leaves
        "/style/theme3.css", // cyan bikini
        "/style/theme4.css", // playful ball thingie
        "/style/theme5.css", // leaning out
        "/style/theme6.css", // perfume
        "/style/theme7.css",  // akb
        "/style/minimal_theme.css" // mobile
    };

    // starts off with sunday as 0 and saturday 6
    protected int[] themePerDay = new int[]
    {
        5, 1, 0, 6, 2, 3, 4
    };

    /// <summary>
    /// Used to override detection (used from configuration in http://site/themes
    /// </summary>
    public string Theme
    {
        get;
        set;
    }

    protected void Page_PreRender(object sender, EventArgs e)
    {
        bool isIPv6 = Globals.IsIPv6();
        ipv6.Visible = isIPv6;

        string themeUrl = string.Empty;
        if (Theme != null)
        {
            themeUrl = Theme;

            int themeNo = 0;
            if (int.TryParse(themeUrl, out themeNo))
            {
                themeUrl = GetThemeFromNumber(themeNo);
            }
        }
        else
        {
            themeUrl = GetThemeUrl();
        }

        SetThemeCss(themeUrl);
    }

    protected void SetThemeCss(string url)
    {
        if (false == string.IsNullOrEmpty(url))
        {
            themeCss.Href = url;
            themeCss.Visible = true;
        }
        else
        {
            themeCss.Visible = false;
        }
    }

    protected string GetThemeUrl()
    {
        string themeUrl = string.Empty;
        int themeNumber = 0;

        HttpCookie coockie = Request.Cookies["theme"];

        if (coockie != null)
        {
            string cTheme = coockie.Values["themeNumber"];
            int.TryParse(cTheme, out themeNumber);

            themeUrl = coockie.Values["themeUrl"];
        }

        if (themeNumber == -1)
        {
            // url set in coockie code
        }
        else
        {
            themeUrl = GetThemeFromNumber(themeNumber);
        }

        return themeUrl;
    }

    protected string GetThemeFromNumber(int number)
    {
        string themeUrl = string.Empty;

        if (number >= 1 && number <= 8)
        {
            themeUrl = themes[--number];
        }
        else
        {
            // any non-documented behaviour should lead to webmaster's choice
            int day = (int)DateTime.Now.DayOfWeek;
            int theme = themePerDay[day];
            themeUrl = themes[theme];
        }

        return themeUrl;
    }
}
