﻿<%@ WebHandler Language="C#" Class="Comments" %>

using System;
using System.Collections.Generic;
using System.Web;
using System.Linq;
using RssComponents;

public class Comments : IHttpHandler
{
    private SayuminDataContext db;
    private IQueryable<Comment> query;
    private string description = "sayumin.org: ";
    private string url;
    
    public void ProcessRequest (HttpContext context) {
        db = new SayuminDataContext();

        int episodeID = -1;
        bool isEpisode = int.TryParse(context.Request.QueryString["EpisodeID"], out episodeID);

        int radioShowID = -1;
        bool isRadioshow = int.TryParse(context.Request.QueryString["RadioshowID"], out radioShowID);

        int newsID = -1;
        bool isNews = int.TryParse(context.Request.QueryString["NewsID"], out newsID);

        //
        context.Response.ContentType = "application/rss+xml";

        string feedtext = "";
        try
        {
            query = from c in db.Comments
                    select c;

            if (isRadioshow)
            {
                query = from c in query
                        where c.RadioshowID == radioShowID
                        select c;

                var radioShow = (from rs in db.RadioShows
                                 where rs.ID == radioShowID
                                 select rs).Single();

                description += radioShow.DisplayName;
                url = "http://www,sayumin.org" + Globals.GetUrl(radioShow);
            }
            else if (isEpisode)
            {
                query = from c in query
                        where c.EpisodeID == episodeID
                        select c;
                
                var episode = (from e in db.Episodes
                               where e.ID == episodeID
                               select e).Single();
                
                description += string.Format("{0} - {1}", episode.RadioShow.DisplayName, episode.Date.ToString("yyyy-MM-dd"));
                url = "http://www,sayumin.org" + Globals.GetUrl(episode);
            }
            else if (isNews)
            {
                query = from c in query
                        where c.NewsID == newsID
                        select c;

                var news = (from n in db.News
                            where n.ID == newsID
                            select n).Single();

                description += string.Format("{0} - {1}", news.Header, news.Date.ToString("yyyy-MM-dd"));
                url = "http://www,sayumin.org" + Globals.GetUrl(news);
            }
            else
            {
                description += "All comments";
                url = "http://www.sayumin.org";
            }

            query = from c in query
                    orderby c.PostDate descending
                    select c;
            
            IEnumerable<Comment> comments = query.Take<Comment>(Globals.ItemsPerFeed);
            
            RssFeed feed = new RssFeed(
				description,
				"http://www.sayumin.org/",
				description,
				"EN-US", url);

            foreach (var comment in comments)
            {
                var title = string.Format("{0} - {1}", comment.RelatedTitle, comment.Nick);
                var commentText = WebToolkit.BBcode.BBdecode(comment.Text);

                var item = new RssItem(title, url, commentText, comment.PostDate);
                feed.Items.Add(item);
            }

            feedtext = feed.ToString();
        }
        catch (Exception)
        { }

        context.Response.Write(feedtext);
    }
 
    public bool IsReusable {
        get {
            return false;
        }
    }

}