﻿<%@ WebHandler Language="C#" Class="Episodes" %>

using System;
using System.Web;
using System.Linq;
using System.Xml.Linq;
using System.Collections.Generic;
using System.Data;
using RssComponents;

public class Episodes : IHttpHandler {
    private const string CONTENT_FORMAT = "<h2>{0}</h2><ul>"
        + "<li><a href=\"{1}\">Download</a></li>"
        + "<li><a href=\"http://www.sayumin.org/stream.ashx?{1}\">Stream</a></li>"
        + "<li><a href=\"{2}\">Comments</a></li>"
        + "</ul><p>Brought to you by <a href=\"http://www.sayumin.org\">sayumin.org</a>.</p>";
        
    public void ProcessRequest (HttpContext context)
    {
        var db = new SayuminDataContext();

        int radioShowID = 0;
        int.TryParse(context.Request.QueryString["ID"], out radioShowID);

        context.Response.ContentType = "application/rss+xml";

        string feedtext = "";
        try
        {

            IQueryable<Episode> query;
            RadioShow radioShow;
            string description = "sayumin.org: ";

            if (radioShowID != 0)
            {
                query = (from e in db.Episodes
                         orderby e.Date descending
                         where e.RadioShowID == radioShowID
                         select e).Take(10);

                radioShow = (from rs in db.RadioShows
                             where rs.ID == radioShowID
                             select rs).Single();

                description += radioShow.DisplayName;
            }
            else
            {
                query = (from e in db.Episodes
                         orderby e.Date descending
                         select e).Take(10);

                description += "All shows";
            }

            RssFeed feed = new RssFeed(
                description,
                "http://www.sayumin.org/",
                description,
                "EN-US", context.Request.Url.AbsoluteUri);

            foreach (var episode in query)
            {
                var episodeUrl = episode.PublicURL;
                episodeUrl = HttpContext.Current.Server.HtmlEncode(episodeUrl).Replace(" ", "%20");
                var title = string.Format("{0} - {1}", episode.RadioShow.DisplayName, episode.Date.ToString("yyyy-MM-dd"));
                var comment = string.Format(CONTENT_FORMAT, title, episodeUrl, "http://www.sayumin.org" + Globals.GetUrl(episode));
                
                var item = new RssItem(title, episodeUrl, comment, episode.FetchDate);

                var enclosure = new RssEnclosure()
                {
                    url = episodeUrl,
                    size = episode.Size
                };

                item.Enclosure = enclosure;

                feed.Items.Add(item);
            }

            feedtext = feed.ToString();
        }
        catch (Exception)
        { }

        context.Response.Write(feedtext);
    }
 
    public bool IsReusable {
        get {
            return true;
        }
    }

}